@@ -2,31 +2,30 @@
 #ifndef __CONFIG_H_
 #define __CONFIG_H_
 
-#include "../acsyscap.h"
+#include "acsyscap.h"
 
-
-#define HAVE_LZMA
-
-
-#define ACVERSION "0.5.6"
+#define ACVERSION "0.5.14"
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
-
+#include <climits>
 
 // keep this consistently, for experiments with other STL implementations
 #define MYSTD std
 //#define MYSTD ustl
 //#include <ustl.h>
 
-#ifdef HAVE_TR1_MEMORY
+#ifdef HAVE_MEMORY_SPTR
+#include <memory>
+#define SMARTPTR_SPACE MYSTD
+#elif defined HAVE_TR1_MEMORY
 #include <tr1/memory>
 #define SMARTPTR_SPACE MYSTD::tr1
 #elif defined HAVE_BOOST_SMARTPTR
 #include <boost/smart_ptr.hpp>
 #define SMARTPTR_SPACE boost
 #else
-#error Unable to find smart pointer implementation, install Boost or recent compiler and STL with TR1 features
+#error Unable to find smart pointer implementation, install Boost or recent compiler with STL containing TR1 components. Set BOOSTDIR in Makefile if needed.
 #endif
 
 #ifndef HAVE_ZLIB
@@ -53,6 +52,9 @@
 #define MYCATCH catch
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
 
 #define ENEMIESOFDOSFS "?[]\\=+<>:;#"
 
