@@ -23,7 +23,7 @@ class tDlJob;
 class dlcon : public lockable, tcpconnect
 { 
     public:
-        dlcon(bool bManualExecution, MYSTD::string *xff=NULL);
+        dlcon(bool bManualExecution, mstring *xff=NULL);
         ~dlcon();
                 
         void WorkLoop();
@@ -32,10 +32,10 @@ class dlcon : public lockable, tcpconnect
 
 	    void AddJob(tFileItemPtr m_pItem, tHttpUrl hi);
         void AddJob(tFileItemPtr m_pItem, 
-        		acfg::tHostiVec *pBackends, const MYSTD::string & sPatSuffix);
+        		acfg::tHostiVec *pBackends, const mstring & sPatSuffix);
     	void EnqJob(tDlJob *);
 
-        MYSTD::string m_sXForwardedFor;
+        mstring m_sXForwardedFor;
         
     private:
     	
@@ -45,7 +45,7 @@ class dlcon : public lockable, tcpconnect
     	MYSTD::list<tDlJob*> m_qToReceive;
     	int m_wakepipe[2];
     	
-    	//MYSTD::string m_sConnectedHost; // used to remember to which host we are connected
+    	//mstring m_sConnectedHost; // used to remember to which host we are connected
 
     	acbuf m_InBuf;
     	
@@ -54,8 +54,7 @@ class dlcon : public lockable, tcpconnect
 
     	bool m_bSingleRun;
     	    
-    	//MYSTD::set<tHttpUrl*> m_MirrorBlacklist;
-        MYSTD::set<MYSTD::string> m_MirrorHostBlacklist;
+    	MYSTD::set<mstring> m_MirrorHostBlacklist;
 
     	//not to be copied
     	dlcon & operator=(const dlcon&);
