@@ -31,7 +31,7 @@ public:
 	virtual ~fileitem();
 	
 	// initialize file item, return the status
-	FiStatus Setup(bool bDynType, bool bIgnoreCached=false);
+	virtual FiStatus Setup(bool bDynType);
 	
 	virtual int GetFileFd();
 	uint64_t GetTransferCount();
@@ -40,10 +40,10 @@ public:
 	
 	// downloader instruments
 	
-	bool DownloadStartedStoreHeader(const header & h, const char *pNextData);
+	virtual bool DownloadStartedStoreHeader(const header & h, const char *pNextData);
 	void AddDownloaderRef();
-	void DelDownloaderRef(const MYSTD::string & sReasonStatusLine);
-	//virtual void SetFailureMode(const MYSTD::string & message, FiStatus fist=FIST_ERROR,
+	void DelDownloaderRef(const mstring & sReasonStatusLine);
+	//virtual void SetFailureMode(const mstring & message, FiStatus fist=FIST_ERROR,
 	//	bool bOnlyIfNoDlRunnuning=false);
 	
 	/*!
@@ -52,35 +52,47 @@ public:
 	virtual bool StoreFileData(const char *data, unsigned int size);
 	header const & GetHeaderUnlocked();
 	inline header GetHeader() { setLockGuard; return m_head; }
-	MYSTD::string GetHttpMsg();
+	mstring GetHttpMsg();
 	
 	FiStatus GetStatus() { setLockGuard; return status; }
 	FiStatus GetStatusUnlocked(off_t &nGoodDataSize) { nGoodDataSize = m_nSizeChecked; return status; }
+	void ResetCacheState();
+	
+	uint64_t m_nIncommingCount;
+	off_t m_nSizeSeen;
+	mstring m_sPath, m_sKey;
 	
 	bool m_bCheckFreshness;
 
-	uint64_t m_nIncommingCount;
-	off_t m_nSizeSeen;
-	MYSTD::string m_sPath, m_sKey;
-	
+	bool m_bHeadOnly; // this is only good for special purposes
+
 protected:
-	fileitem(MYSTD::string);
+
+	bool m_bAllowStoreData;
+
+	fileitem(mstring);
 	off_t m_nSizeChecked;
 	header m_head;
 	int m_filefd;
 	int m_nDlRefsCount;
 	FiStatus status;
 
-	bool m_bAllowStoreData;
-
-
 	////////////////////////////////
 	// Item pool management methods
 	////////////////////////////////
 public:
 	// public constructor wrapper, get a unique object from the map
-	static tFileItemPtr GetFileItem(MYSTD::string sPath); // also opens the cached file and increases reference count
+	static tFileItemPtr GetFileItem(mstring sPath); // also opens the cached file and increases reference count
 	virtual void Unreg(); // decrease dependent user count
+
+	//! Call Unreg directly, or return immediatly but call it few seconds later
+	static void UnregOneASAP(tFileItemPtr);
+	//! @return: true iff there is still something in the pool for later cleaning
+	static time_t DoDelayedUnregAndCheck();
+
+private:
+	time_t m_nTimeExpireAt; //! when to consider this item obsolete
+	static time_t m_nEarliestExpiration; //! when to consider a closer look for expired items
 };
 
 extern lockable mapLck;
