@@ -4,6 +4,7 @@
 
 #include "config.h"
 #include "acbuf.h"
+#include "fileio.h"
 
 #ifdef HAVE_LIBBZ2
 #include <bzlib.h>
@@ -32,31 +33,29 @@ public:
 	 * 			filename w/o suffix or path prefix back into it
 	 * @param bCriticalOpen Terminate program on failure
 	 */ 
-	bool OpenFile(const MYSTD::string & sFilename, bool bNoMagic=false);
+	bool OpenFile(const mstring & sFilename, bool bNoMagic=false);
 	
-	MYSTD::string GetPureFilename();
+	mstring GetPureFilename();
 	
 	//////! Filename with all prepended path and compressed suffix stripped
-	//////void GetBaseFileName(MYSTD::string & sOut);
+	//////void GetBaseFileName(mstring & sOut);
 	//! Returns lines when beginning with non-space, otherwise empty string. 
 	//! @return False on errors.
-	bool GetOneLine(MYSTD::string & sOut);
-	MYSTD::string GetPositionDescription() { return (tSS() << m_sOrigName << ":" << m_nCurLine); }
+	bool GetOneLine(mstring & sOut);
+	mstring GetPositionDescription() { return (tSS() << m_sOrigName << ":" << m_nCurLine); }
 	bool CheckGoodState(bool bTerminateOnErrors) const;
 	
-	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], 
-			bool bTryUnpack, off_t &scannedSize);
-	
-	bool GetSha1Sum(uint8_t out[], off_t &scannedSize, const char *szUnpStoreFile=NULL);
-	
-	static bool GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], 
-			bool bTryUnpack, off_t &scannedSize,
-			const char *szUnpStoreFile=NULL);
-	
-	static bool GetMd5String(const MYSTD::string & sFileName, char out[]);
-	static bool CheckMd5Sum(const MYSTD::string & sFileName, const MYSTD::string & sReference);
-	inline const char *GetBuffer() { return m_szFileBuf; };
-	inline size_t GetSize() { return m_nBufSize; };
+	bool GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE *pDumpFile=NULL);
+	static bool GetChecksum(const mstring & sFileName, int csType, uint8_t out[],
+			bool bTryUnpack, off_t &scannedSize, FILE *pDumpFile=NULL);
+
+	/*
+	static bool GetMd5String(const mstring & sFileName, char out[]);
+	static bool CheckMd5Sum(const mstring & sFileName, const mstring & sReference);
+	*/
+
+	inline const char *GetBuffer() const { return m_szFileBuf; };
+	inline size_t GetSize() const { return m_nBufSize; };
 	void Close();
 
 private:
@@ -68,7 +67,7 @@ private:
 	acbuf m_UncompBuf; // uncompressed window
 	
 	// visible position reporting
-	MYSTD::string m_sOrigName;
+	mstring m_sOrigName;
 	UINT m_nCurLine;
 	
 	int m_fd;
@@ -94,15 +93,23 @@ private:
 
 extern uint_fast16_t hexmap[];
 
-inline bool CsEqual(CUCHAR a[], uint8_t b[], unsigned short binLen)
+inline bool CsEqual(const char *sz, uint8_t b[], unsigned short binLen)
 {
+	CUCHAR *a=(CUCHAR*)sz;
+	if(!a)
+		return false;
 	for(int i=0; i<binLen;i++)
 	{
+		if(!*a)
+			return false;
+
 		uint_fast16_t r=hexmap[a[i*2]] * 16 + hexmap[a[i*2+1]];
 		if(r != b[i]) return false;
 	}
 	return true;
 };
 
+bool Bz2compressFile(const char *pathIn, const char*pathOut);
+
 #endif // __FILEREADER_H
 
