@@ -19,9 +19,15 @@ protected:
 	tHttpUrl * m_proxy;
 	int m_conFd;
 
-	bool _Connect(const MYSTD::string & sHostname, MYSTD::string &sErrOut);
+	bool _Connect(const mstring & sHostname, mstring &sErrOut);
 	void _Disconnect();
 
+private:
+	tcpconnect operator=(const tcpconnect&);
+	tcpconnect(const tcpconnect&);
+
+protected:
+	acfg::tHostiVec::IHookHandler *m_pConnStateObserver;
 };
 
 #endif /* TCPCONNECT_H_ */
