@@ -82,3 +82,47 @@ int acbuf::sysread(int fd) {
     return(n);
 }
 
+inline char unEscape(const char p)
+{
+	switch (p)
+	{
+	case '0':
+		return '\0';
+	case 'a':
+		return '\a';
+	case 'b':
+		return '\b';
+	case 't':
+		return '\t';
+	case 'n':
+		return '\n';
+	case 'r':
+		return '\r';
+	case 'v':
+		return '\v';
+	case 'f':
+		return '\f';
+	default:
+		return p;
+	}
+}
+
+tSS & tSS::addEscaped(const char *fmt)
+{
+	if(!fmt || !*fmt)
+		return *this;
+	int nl=strlen(fmt);
+	reserve(length()+nl);
+	char *p=wptr();
+
+	for(;*fmt;fmt++)
+	{
+		if(*fmt=='\\')
+			*(p++)=unEscape(*(++fmt));
+		else
+			*(p++)=*fmt;
+	}
+	got(p-wptr());
+	return *this;
+}
+
