@@ -7,6 +7,8 @@
 #include "meta.h"
 #include "filereader.h"
 #include "fileio.h"
+#include "lockable.h"
+#include "cleaner.h"
 
 #include <regex.h>
 
@@ -19,6 +21,14 @@
 
 using namespace MYSTD;
 
+
+#define _iterPos(it, start) (it-start.begin())/sizeof(it)
+#define sProblemLoc szPath<< ':'<< _iterPos(it, lines)
+
+string sFilterSet(SPACECHARS "#");
+#define IsValidButIrrelevantLine(x) (x.empty() || stmiss != sFilterSet.find(x[0]))
+#define BARF(x) { cerr << x << endl; exit(EXIT_FAILURE); }
+
 namespace rechecks
 {
 bool CompileExpressions();
@@ -28,15 +38,18 @@ bool CompileUncExpressions(const string & req, const string & tgt);
 
 namespace acfg {
 
+
 // internal stuff:
 char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 string sPopularPath("/debian/");
 
 string dontcache, dontcacherq, dontcachetgt;
 
-tStringMap localdirs;
+tStrMap localdirs;
 NoCaseStringMap mimemap;
 
+#ifndef MINIBUILD
+
 MapNameToString n2sTbl[] = {
 		{  "Port", 			&port , 0}
 		,{ "CacheDir", 	&cachedir, 0 }
@@ -54,6 +67,8 @@ MapNameToString n2sTbl[] = {
 		,{ "DontCache",	&dontcache, 0}
 		,{ "DontCacheRequested",	&dontcachetgt, 0}
 		,{ "DontCacheResolved",	&dontcacherq, 0}
+		,{ "PrecacheFor", &mirrorsrcs, 0}
+		,{ "RequestAppendix", &requestapx, 0}
 };
 
 MapNameToInt n2iTbl[] = {
@@ -83,13 +98,6 @@ MapNameToInt n2iTbl[] = {
 		,{ "FilePerms", &fileperms, 0, 8}
 };
 
-#define _iterPos(it, start) (it-start.begin())/sizeof(it)
-#define sProblemLoc szPath<< ':'<< _iterPos(it, lines)
-
-string sFilterSet(SPACECHARS "#");
-#define IsValidButIrrelevantLine(x) (x.empty() || stmiss != sFilterSet.find(x[0]))
-#define BARF(x) { cerr << x << endl; exit(EXIT_FAILURE); }
-
 void _ReadRewriteFiles(const string & sFile, const string & sRepName);
 void _ReadBackendsFiles(const string & sFile, const string &sRepName);
 
@@ -149,7 +157,7 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 	filereader reader;
 	reader.OpenFile(szFilename);
 	reader.CheckGoodState(true);
-	string sLine, key, val;
+	string sLine;
 	for (bool bNotEof=true; bNotEof;)
 	{
 		bNotEof=reader.GetOneLine(sLine);
@@ -158,10 +166,12 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 #ifdef DEBUG
 		cerr << sLine <<endl;
 #endif
+		tStrPos pos=sLine.find('#');
+		if(stmiss != pos)
+			sLine.erase(pos);
+
 		if(! SetOption(sLine))
 			BARF("Error reading main options, terminating.");
-		// XXX: move that to SetOption... if (debug>4)
-		//	cout << key << " -> "<< val <<endl;
 	}
 	return true;
 }
@@ -175,7 +185,7 @@ const string * _CheckBEentryGetNamePtr(const string & sRepName)
 			itHostiVec = mapRepName2Backends.find(sRepName);
 	if (itHostiVec == mapRepName2Backends.end())
 	{
-		mapRepName2Backends[sRepName]=tHostiVec(0);
+		mapRepName2Backends[sRepName]=tHostiVec();
 		itHostiVec=mapRepName2Backends.find(sRepName); // must refer to that string on the heap
 		if (debug>4)
 			cout << "created empty backend entry for "<< sRepName <<endl;
@@ -218,18 +228,123 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 	}
 }
 
-inline string _GetBase64Auth(const string & sUserColonPass)
+inline bool ParseOptionLine(const string &sLine, string &key, string &val, bool bQuiet)
 {
-	int cols, bits, c, char_count;
-
-	char_count = 0;
-	bits = 0;
-	cols = 0;
-	tStrPos pos(0);
-	string out;
-	while ( pos<sUserColonPass.size())
+	string::size_type posCol = sLine.find(":");
+	string::size_type posEq = sLine.find("=");
+	if (posEq==stmiss && posCol==stmiss)
 	{
-		c=sUserColonPass[pos++];
+		if(!bQuiet)
+			cerr << "Not a valid configuration directive: " << sLine <<endl;
+		return false;
+	}
+	string::size_type pos;
+	if (posEq!=stmiss && posCol!=stmiss)
+		pos=min(posEq,posCol);
+	else if (posEq!=stmiss)
+		pos=posEq;
+	else
+		pos=posCol;
+
+	key=sLine.substr(0, pos);
+	val=sLine.substr(pos+1);
+	trimString(key);
+	trimString(val);
+	if(key.empty())
+		return false; // weird
+
+	return true;
+}
+
+
+struct tHookHandler: public tHostiVec::IHookHandler, public lockable
+{
+	string cmdRel, cmdCon;
+	time_t downDuration, downTimeNext;
+
+	int m_nRefCnt;
+
+	tHookHandler(cmstring&) :
+		downDuration(30), downTimeNext(0), m_nRefCnt(0)
+	{
+//		cmdRel = "logger JobRelease/" + name;
+//		cmdCon = "logger JobConnect/" + name;
+	}
+	virtual void JobRelease()
+	{
+		setLockGuard;
+		if (0 >= --m_nRefCnt)
+		{
+			//system(cmdRel.c_str());
+			downTimeNext= time(0) + downDuration;
+		}
+	}
+	virtual void JobConnect()
+	{
+		setLockGuard;
+		if (0 == m_nRefCnt++)
+		{
+			if(downTimeNext) // huh, already ticking? reset
+				downTimeNext=0;
+			else
+				system(cmdCon.c_str());
+		}
+	}
+};
+
+void _AddHooksFile(cmstring& vname)
+{
+	filereader reader;
+	if(!reader.OpenFile(acfg::confdir+"/"+vname+".hooks"))
+		return;
+
+	string sLine, key, val;
+	struct tHookHandler &hs = *(new tHookHandler(vname));
+
+	for (bool bNotEof=true; bNotEof;)
+	{
+		bNotEof=reader.GetOneLine(sLine);
+		if (IsValidButIrrelevantLine(sLine))
+			continue;
+
+		if(!ParseOptionLine(sLine, key, val, false))
+			continue;
+		const char *p=key.c_str();
+		if(strcasecmp("PreUp", p) == 0)
+		{
+			hs.cmdCon = val;
+		}
+		else if(strcasecmp("Down", p) == 0)
+		{
+			hs.cmdRel = val;
+		}
+		else if(strcasecmp("DownTimeout", p) == 0)
+		{
+			errno=0;
+			uint n = strtoul(val.c_str(), NULL, 10);
+			if(!errno)
+				hs.downDuration = n;
+		}
+	}
+	mapRepName2Backends[vname].m_pHooks = &hs;
+
+}
+
+string _GetBase64Auth(const string & s)
+{
+	int cols=0, bits=0, c=0, char_count=0;
+	tStrPos pos=0;
+	string out;
+	while ( pos<s.size() )
+	{
+		c=s[pos++];
+		if('%' == c && pos<s.length()-1
+				&& hexmap[(unsigned)s[pos]]>=0
+				&& hexmap[(unsigned)s[pos+1]]>=0)
+		{
+			c=char(16*hexmap[(unsigned)s[pos]]+hexmap[(unsigned)s[pos+1]]);
+			pos+=2;
+		}
 		bits += c;
 		char_count++;
 		if (char_count == 3)
@@ -266,31 +381,60 @@ inline string _GetBase64Auth(const string & sUserColonPass)
 	return out;
 }
 
+inline void _ParseLocalDirs(cmstring &value)
+{
+	tStrVec tokens;
+	Tokenize(value, ";", tokens);
+	for(tStrVecIter it=tokens.begin(); it!=tokens.end(); it++)
+	{
+		trimString(*it);
+		tStrPos pos = it->find_first_of(SPACECHARS);
+		if(stmiss == pos)
+		{
+			cerr << "Cannot map " << *it << ", needed format: virtualdir realdir, ignoring it";
+			continue;
+		}
+		string from=it->substr(0, pos);
+		trimString(from, "/");
+		string what=it->substr(pos);
+		trimString(what, SPACECHARS "'\"");
+		if(what.empty())
+		{
+			cerr << "Unsupported target of " << from << ": " << what << ", ignoring it" << endl;
+			continue;
+		}
+		localdirs[from]=what;
+	}
+}
+
+inline void _FetchMimeTypes()
+{
+	filereader mreader;
+	tStrVec toks;
+	string line;
+	for(mreader.OpenFile("/etc/mime.types"); mreader.CheckGoodState(false); )
+	{
+		if(!mreader.GetOneLine(line))
+			break;
+		if(IsValidButIrrelevantLine(line))
+			continue;
+
+		if(Tokenize(line, SPACECHARS, toks, false)>1)
+		{
+			for(UINT i=1; i<toks.size();i++)
+				mimemap[toks[i]]=toks[0];
+		}
+	}
+}
+
 bool SetOption(const string &sLine, bool bQuiet)
 {
-	string::size_type posCol = sLine.find(":");
-	string::size_type posEq = sLine.find("=");
-	if (posEq==stmiss && posCol==stmiss)
-	{
-		if(!bQuiet)
-			cerr << "Not a valid configuration directive: " << sLine <<endl;
-		return false;
-	}
-	string::size_type pos;
-	if (posEq!=stmiss && posCol!=stmiss)
-		pos=min(posEq,posCol);
-	else if (posEq!=stmiss)
-		pos=posEq;
-	else
-		pos=posCol;
 
-	string key=sLine.substr(0, pos);
-	string value=sLine.substr(pos+1);
-	trimString(key);
-	trimString(value);
-	if(key.empty())
-		return false; // weird
-	
+	string key, value;
+
+	if(!ParseOptionLine(sLine, key, value, bQuiet))
+		return false;
+
 	string * psTarget;
 	int * pnTarget;
 	int nNumBase(10);
@@ -338,11 +482,12 @@ bool SetOption(const string &sLine, bool bQuiet)
 			cerr << "Invalid proxy specification, aborting..." << endl;
 			exit(EXIT_FAILURE);
 		}
-		tStrPos pos = proxy_info.sHost.find('@');
+		tStrPos pos = proxy_info.sHost.rfind('@');
 		if (stmiss != pos)
 		{
+			tStrPos pwStart= startsWithSz(value, "http://") ? 7 : 0;
 			proxy_info.sPath = string("Proxy-Authorization: Basic ")
-					+ _GetBase64Auth(value.substr(0, pos)) + "\r\n";
+					+ _GetBase64Auth(value.substr(pwStart, pos)) + "\r\n";
 			proxy_info.sHost.erase(0, pos + 1);
 		}
 		else
@@ -369,46 +514,8 @@ bool SetOption(const string &sLine, bool bQuiet)
 	}
 	else if(0==strcasecmp(key.c_str(), "LocalDirs"))
 	{
-		tStrVec tokens;
-		Tokenize(value, ";", tokens);
-		for(tStrVecIter it=tokens.begin(); it!=tokens.end(); it++)
-		{
-			trimString(*it);
-			tStrPos pos = it->find_first_of(SPACECHARS);
-			if(stmiss == pos)
-			{
-				cerr << "Cannot map " << *it << ", needed format: virtualdir realdir, ignoring it";
-				continue;
-			}
-			string from=it->substr(0, pos);
-			trimString(from, "/");
-			string what=it->substr(pos);
-			trimString(what, SPACECHARS "'\"");
-			if(what.empty())
-			{
-				cerr << "Unsupported target of " << from << ": " << what << ", ignoring it" << endl;
-				continue;
-			}
-			localdirs[from]=what;
-		}
-
-		filereader mreader;
-		tStrVec toks;
-		string line;
-		for(mreader.OpenFile("/etc/mime.types"); mreader.CheckGoodState(false); )
-		{
-			if(!mreader.GetOneLine(line))
-				break;
-			if(IsValidButIrrelevantLine(line))
-				continue;
-
-			if(Tokenize(line, SPACECHARS, toks, false)>1)
-			{
-				for(UINT i=1; i<toks.size();i++)
-					mimemap[toks[i]]=toks[0];
-			}
-		}
-
+		_ParseLocalDirs(value);
+		_FetchMimeTypes();
 		return !localdirs.empty();
 	}
 	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
@@ -435,7 +542,7 @@ bool SetOption(const string &sLine, bool bQuiet)
 				break;
 			_AddRemapInfo(bIsBackend, tokens[i], vname);
 		}
-		
+		_AddHooksFile(vname);
 	}
 	else
 	{
@@ -682,6 +789,14 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 }
 
 
+tHostiVec::~tHostiVec()
+{
+	if(m_pHooks)
+	{
+		delete m_pHooks;
+		m_pHooks=NULL;
+	}
+}
 
 void ReadConfigDirectory(const char *szPath)
 {
@@ -762,6 +877,8 @@ void PostProcConfig()
 	for(tStrPos pos; stmiss != (pos = cachedir.find(SZPATHSEP SZPATHSEP )); )
 		cachedir.erase(pos, 1);
 
+	cacheDirSlash=cachedir+CPATHSEP;
+
    if(!pidfile.empty() && pidfile.at(0) != CPATHSEP)
    {
 	   cerr << "Pid file path must be absolute, terminating..."  <<endl;
@@ -842,48 +959,120 @@ void PostProcConfig()
 		   const_cast<string&>(suc->first.sPath)=it->first.sPath;
    }
    */
+
+#if 0 //def DEBUG
+#warning adding hook control pins
+   for(tMapString2Hostivec::iterator it = mapRepName2Backends.begin();
+		   it!=mapRepName2Backends.end() ; ++it)
+   {
+	   tHookHandler *p = new tHookHandler(it->first);
+	   p->downDuration=10;
+	   p->cmdCon = "logger wanna/connect";
+	   p->cmdRel = "logger wanna/disconnect";
+	   it->second.m_pHooks = p;
+   }
+#endif
+
+   if(debug == -42)
+   {
+	   /*
+	   for(tMapString2Hostivec::const_iterator it=mapRepName2Backends.begin();
+			   it!=mapRepName2Backends.end(); it++)
+	   {
+		   for(tHostiVec::const_iterator jit=it->second.begin();
+				   jit != it->second.end(); jit++)
+		   {
+			   cout << jit->ToURI() <<endl;
+		   }
+	   }
+	   */
+	   for(tUrl2RepIter it=mapUrl2pVname.begin(); it!=mapUrl2pVname.end(); it++)
+	   {
+		   cout << it->first.ToURI() << " ___" << *(it->second) << endl;
+	   }
+
+	   exit(1);
+   }
+} // PostProcConfig
+
+time_t ExecutePostponed()
+{
+	time_t ret(cleaner::never), now(time(0));
+	for (tMapString2Hostivec::iterator it = mapRepName2Backends.begin(); it
+			!= mapRepName2Backends.end(); ++it)
+	{
+		if (!it->second.m_pHooks)
+			continue;
+		tHookHandler & hooks = *(static_cast<tHookHandler*> (it->second.m_pHooks));
+		lockguard g(hooks);
+		if (hooks.downTimeNext)
+		{
+			if (hooks.downTimeNext <= now) // time to execute
+			{
+				if(acfg::debug>1)
+					aclog::misc(hooks.cmdRel, 'X');
+				if(acfg::debug>3)
+					aclog::flush();
+
+				system(hooks.cmdRel.c_str());
+				hooks.downTimeNext = 0;
+			}
+			else // in future, use the soonest time
+				ret = min(ret, hooks.downTimeNext);
+		}
+	}
+	return ret;
 }
-}
+
+#endif // MINIBUILD
+
+} // namespace acfg
 
 namespace rechecks
 {
-regex_t reIfiles, rePfiles, reWfiles;
-deque<regex_t> vecReqPatters, vecTgtPatterns;
+	cmstring *psRex[] = {&acfg::vfilepat, &acfg::pfilepat, &acfg::wfilepat};
+	regex_t rex[_countof(psRex)];
 
 bool CompileExpressions()
 {
-	//printf("%s\\n%s\\n", a, b);
-
-    if(regcomp(&rePfiles, acfg::pfilepat.c_str(), REG_EXTENDED))
-    	return false;
-    if(regcomp(&reIfiles, acfg::vfilepat.c_str(), REG_EXTENDED))
-    	return false;
-    if(regcomp(&reWfiles, acfg::wfilepat.c_str(), REG_EXTENDED))
-    	return false;
-
-    return true;
+	char buf[1024];
+	for(UINT i=0; i<_countof(psRex); i++)
+	{
+		int nErr=regcomp(&rex[i], psRex[i]->c_str(), REG_EXTENDED);
+		if(nErr)
+		{
+			regerror(nErr,  &rex[i], buf, sizeof(buf));
+			buf[_countof(buf)-1]=0; // better be safe...
+			std::cerr << buf << ": " << *(psRex[i]) << std::endl;
+			return false;
+		}
+	}
+	return true;
 }
 
 eFileKind GetFiletype(const string & in) {
-	LOGSTART("rechecks::getFiletype");
-    dbgline;
-    if(!regexec(&reIfiles, in.c_str(), 0, NULL, 0))
+	//LOGSTART("rechecks::getFiletype");
+    //dbgline;
+    if(!regexec(&rex[0], in.c_str(), 0, NULL, 0))
         return FILE_INDEX;
-    dbgline;
-    if(!regexec(&rePfiles, in.c_str(), 0, NULL, 0))
+    //dbgline;
+    if(!regexec(&rex[1], in.c_str(), 0, NULL, 0))
         return FILE_PKG;
-    dbgline;
+    //dbgline;
     return FILE_INVALID;
 }
 
 bool MatchWhitelist(const string & in)
 {
 	LOGSTART("MatchWhiteList");
-	bool bPersistent=!regexec(&reWfiles, in.c_str(), 0, NULL, 0);
+	bool bPersistent=!regexec(&rex[2], in.c_str(), 0, NULL, 0);
 	LOG(in <<" is " << (bPersistent ? "persistent" : "mortal") <<  " file");
 	return bPersistent;
 }
 
+deque<regex_t> vecReqPatters, vecTgtPatterns;
+
+#ifndef MINIBUILD
 inline bool CompileUncachedRex(const string & token, bool bIsTgtPattern, bool bRecursiveCall)
 {
 	deque<regex_t> & patvec = bIsTgtPattern ? vecTgtPatterns : vecReqPatters;
@@ -970,16 +1159,42 @@ bool MatchUncacheableTarget(const string &in)
 	return false;
 }
 
+#endif //MINIBUILD
+
+
+} // namespace rechecks
+
+mstring GetDirPart(const string &in)
+{
+	if(in.empty())
+		return "";
+
+	tStrPos end = in.find_last_of(CPATHSEP);
+	if(end == stmiss) // none? don't care then
+		return "";
+
+	return in.substr(0, end+1);
 }
 
 void mkbasedir(const string & path)
 {
-    for(UINT pos=0; pos<path.size(); pos=path.find("/", pos+1))
+	if(0==mkdir(GetDirPart(path).c_str(), acfg::dirperms) || EEXIST == errno)
+		return; // should succeed in most cases
+
+	UINT pos=0; // but skip the cache dir components, if possible
+	if(startsWith(path, acfg::cacheDirSlash))
+	{
+		// pos=acfg::cachedir.size();
+		pos=path.find("/", acfg::cachedir.size()+1);
+	}
+    for(; pos<path.size(); pos=path.find(SZPATHSEP, pos+1))
     {
         if(pos>0)
             mkdir(path.substr(0,pos).c_str(), acfg::dirperms);
     }
 }
+
+
 /*
 int main(int argc, char **argv)
 {
