@@ -11,8 +11,8 @@ using namespace MYSTD;
 namespace acfg {
 
 #if defined(__CYGWIN32__) || defined(__CYGWIN__)
-#warning FIXME: something is wrong with its regexp and the begining of the vfilepat pattern
-#define VPATPREFIX "" 
+#warning Cygwin seems to have an ancient regex engine which does not distuinguish between greedy and non-greedy repetition, and defaults to non-greedy
+#define INFOLDER "(^|.*/)"
 #else
 #define INFOLDER "(^|.*?/)"
 #endif
@@ -20,7 +20,7 @@ namespace acfg {
 
 // always the same
 string cachedir("/var/tmp"), logdir("/var/tmp"), fifopath, pidfile, reportpage,
-confdir, adminauth, bindaddr;
+confdir, adminauth, bindaddr, mirrorsrcs;
 
 #define COMPRLIST "(\\.gz|\\.bz2|\\.lzma|\\.xz)"
 #define COMPONENT_OPTIONAL "(-[a-z0-9-])"
@@ -38,14 +38,14 @@ string vfilepat( INFOLDER
 		"(Index|Packages"COMPRLIST"?|Release|Release\\.gpg|"
 		"Sources"COMPRLIST"?|release|index\\.db-.*\\.gz|Contents-[^/]*"COMPRLIST
 		"|pkglist[^/]*\\.bz2|rclist[^/]*\\.bz2|/meta-release[^/]*|Translation[^/]*"COMPRLIST
-		"MD5SUMS|SHA1SUMS" // d-i stuff
+		"|MD5SUMS|SHA1SUMS" // d-i stuff
 		"|((setup|setup-legacy)(\\.ini|\\.bz2|\\.hint)(\\.sig)?)|mirrors\\.lst" // cygwin
 		"|repo(index|md)\\.xml(\\.asc|\\.key)?|directory\\.yast" // opensuse
 		"|products|content(\\.asc|\\.key)?|media" // opensuse 2, are they important?
 		"|packages\\.[a-zA-Z][a-zA-Z]\\.gz|info\\.txt|license\\.tar\\.gz|license\\.zip" //opensuse
 		"|.*\\.db\\.tar\\.gz|.*\\.files\\.tar\\.gz|.*\\.abs\\.tar\\.gz" // Arch Linux
 //    "|metalink\\?repo"     "|.*prestodelta\\.xml\\.gz" // fedora core 13, worked in the beginning and then decided to hate me
-		")$" // end of INFOLDER, some specific URLs?
+		")$" // end of filename paterns. Any other some specific URLs?
 		"|/dists/.*/installer-[^/]+/[^0-9][^/]+/images/.*"); // d-i stuff but not containing a number (year) in the revision directory (like "current", "beta", ...)
 
 //string wfilepat( VPATPREFIX  "(Release|Release\\.gpg|release|meta-release|Translation[^/]*\\.bz2)$");
@@ -53,7 +53,7 @@ string vfilepat( INFOLDER
 string wfilepat(INFOLDER
 		"(Release|Release\\.gpg"
 		"|(Packages|Sources)"COMPRLIST"?"
-		"|Translation[^/]*"COMPRLIST
+		"|Translation[^/]*"COMPRLIST // to be checked, but they should never really go anywhere
 		"|MD5SUMS|SHA1SUMS"
 		"|.*\\.xml" // SUSE
 		"|.*\\.db\\.tar\\.gz|.*\\.files\\.tar\\.gz|.*\\.abs\\.tar\\.gz" // Arch Linux
@@ -72,6 +72,8 @@ string agentname("Debian Apt-Cacher-NG/" ACVERSION);
 string remoteport("80"), port("3142");
 string agentheader;
 
+string requestapx("Cache-Control: no-store,no-cache,max-age=0\\r\\n");
+
 #ifdef DEBUG
 int debug(3), foreground(true);
 //string cachedir("/var/cache/acng"), logdir("/var/log/acng"), fifopath, pidfile;
@@ -81,4 +83,6 @@ int debug(0), foreground(false);
 
 tHttpUrl proxy_info; // abuses path member for auth data
 
+string cacheDirSlash; // guaranteed to have a trailing path separator
+
 }
