@@ -9,6 +9,7 @@
 #include "dlcon.h"
 #include "acbuf.h"
 #include "tcpconnect.h"
+#include "cleaner.h"
 
 #include <sys/select.h>
 #include <signal.h>
@@ -18,6 +19,7 @@
 
 using namespace MYSTD;
 
+
 con::con(int fdId, const char *c) :
 	m_confd(fdId),
     m_bStopActivity(false),
@@ -26,13 +28,15 @@ con::con(int fdId, const char *c) :
 {
 	if(c) // if NULL, pick up later when sent by the wrapper
 		m_sClientHost=c;
-	
+
     LOGSTART2("con::con", "fd: " << fdId << ", clienthost: " << c);
 
-    
+    // actually, just make sure that it runs right before, even if the cleaner thread is busy
+    fileitem::DoDelayedUnregAndCheck();
+
 #ifdef KILLABLE
     wakepipe[0]=wakepipe[1]=-1;
-    
+
     if(pipe(wakepipe) == 0) {
     	set_nb(wakepipe[0]);
     	set_nb(wakepipe[1]);
@@ -40,7 +44,7 @@ con::con(int fdId, const char *c) :
     else
     	m_bStopActivity=true;
 #endif
-    
+
 };
 
 #ifdef KILLABLE
@@ -52,9 +56,6 @@ void con::SignalStop() {
 }
 #endif
 
-
-//void DumpItems();
-
 con::~con() {
 	LOGSTART("con::~con (Destroying connection...)");
 	flushShutdownClose(m_confd);
@@ -84,6 +85,9 @@ con::~con() {
     }
     				
     aclog::flush();
+
+    // some extra work might have been postponed
+    g_victor.Do();
 }
 
 class SoapPassThrough : public tcpconnect
@@ -292,7 +296,7 @@ void con::WorkLoop() {
 				j->PrepareDownload();
 				m_jobs2send.push_back(j);
 			}
-        	MYCATCH(bad_alloc)
+        	MYCATCH(bad_alloc&)
         	{
         		return;
         	}
@@ -317,8 +321,10 @@ void con::WorkLoop() {
 					pjSender=NULL;
 		
 					ldbg("Remaining jobs to send: " << m_jobs2send.size());
+					break;
 				}
 				case(job::R_AGAIN):
+						break;
 				default:
 					break;
 			}
@@ -356,7 +362,7 @@ bool con::SetupDownloader(const char *pszOrigin)
 		if(!m_pDlClient)
 			return false;
 	}
-	MYCATCH(MYSTD::bad_alloc)
+	MYCATCH(bad_alloc&)
 	{
 		return false;
 	}
