@@ -21,14 +21,12 @@ dlcon::dlcon(bool bManualExecution, string *xff) :
 m_bSingleRun(bManualExecution)
 {
 	LOGSTART("dlcon::dlcon");
+	m_wakepipe[0]=m_wakepipe[1]=-1;
 	if (0==pipe(m_wakepipe))
 	{
 		set_nb(m_wakepipe[0]);
 		set_nb(m_wakepipe[1]);
 	}
-	else
-		m_wakepipe[0]=m_wakepipe[1]=-1;
-	
 	if(xff)
 		m_sXForwardedFor=*xff;
 }
@@ -150,7 +148,7 @@ void dlcon::WorkLoop()
 				m_InBuf.clear();
 				sendBuf.clear();
 				bStopRequesting=false;
-				// just be sure about that, not only in _Reconnect
+				// just be sure about that
 				_Disconnect();
 								
 				/* fresh state and/or going for reconnection. Pick up a working hostname and
@@ -168,11 +166,19 @@ void dlcon::WorkLoop()
 				}
 				
 				const string sTargetHost = pj->GetPeerName();
+
+				acfg::tHostiVec::IHookHandler *pobs = pj->GetConnStateObserver();
+				if(pobs)
+					pobs->JobConnect();
+
 				__lockguard.unLock();
 				// might override it with proxy stuff
 				bool bOk=_Connect(sTargetHost, sErrorMsg);
+
 				ldbg("Connection ok? "<<bOk<<", remember host? "<<sTargetHost);
 
+				m_pConnStateObserver=pobs;
+
                 if(bOk)
 					sCurrentlyConnectedHost=sTargetHost;
 				
@@ -256,7 +262,7 @@ void dlcon::WorkLoop()
 			FD_SET(m_conFd, &rfds);
 			nMaxFd=std::max(m_conFd, nMaxFd);
 			
-			if(!sendBuf.empty())
+			if(sendBuf.size())
 			{
 				ldbg("Needs to send " << sendBuf.size() << " bytes")
 				FD_SET(m_conFd, &wfds);
@@ -345,7 +351,7 @@ void dlcon::WorkLoop()
 			}
 		}
 
-		if (m_conFd>=0 && !sendBuf.empty() && FD_ISSET(m_conFd, &wfds))
+		if (m_conFd>=0 && sendBuf.size() && FD_ISSET(m_conFd, &wfds))
 		{
 			FD_CLR(m_conFd, &wfds);
 			
