@@ -9,7 +9,6 @@ using namespace MYSTD;
 
 tDlJob::tDlJob(dlcon *p, tFileItemPtr pFi, const string & sHost,
 		const string & sPath) :
-			m_sRequestCmd("GET "),
 			m_pHostiVec(NULL),
 			m_pStorage(pFi), 
 			m_parent(p),
@@ -20,7 +19,6 @@ tDlJob::tDlJob(dlcon *p, tFileItemPtr pFi, const string & sHost,
 
 tDlJob::tDlJob(dlcon *p, tFileItemPtr pFi, acfg::tHostiVec * pBackends,
 		const MYSTD::string & sPath) :
-			m_sRequestCmd("GET "),
 	m_pHostiVec(pBackends), m_pStorage(pFi),
 			m_parent(p),
 			m_fileUri("", sPath)
@@ -37,6 +35,11 @@ tDlJob::~tDlJob()
 	}
 }
 
+acfg::tHostiVec::IHookHandler * tDlJob::GetConnStateObserver()
+{
+	return m_pHostiVec ? m_pHostiVec->m_pHooks : NULL;
+}
+
 void tDlJob::_Init()
 {
 	m_nRest=0;
@@ -120,7 +123,8 @@ bool tDlJob::AppendRequest(string sForThisHostOnly, tSS &head)
 		return false;
 	}
 
-	head << m_sRequestCmd << (m_parent->m_proxy ? RemoteUri() : RemotePath()) << " HTTP/1.1\r\n"
+	head << (m_pStorage->m_bHeadOnly ? "HEAD " : "GET ")
+			<< (m_parent->m_proxy ? RemoteUri() : RemotePath()) << " HTTP/1.1\r\n"
 	"Host: " << GetPeerName() << "\r\nConnection: keep-alive\r\n";
 	
 	if (m_parent->m_proxy) // also add authorization if there is any
@@ -174,9 +178,11 @@ bool tDlJob::AppendRequest(string sForThisHostOnly, tSS &head)
 	if(acfg::exporigin && !m_parent->m_sXForwardedFor.empty())
 		head << "X-Forwarded-For: " << m_parent->m_sXForwardedFor << "\r\n";
 	
+	if(!acfg::requestapx.empty())
+		head.addEscaped(acfg::requestapx.c_str());
 	head << "\r\n";
 			
-	ldbg("Request cooked: " << head);
+	ldbg("Request cooked, buffer contents: " << head);
 	
 	bRequestPrepared=true;
 	return true;
@@ -276,8 +282,10 @@ tDlJob::tDlResult tDlJob::ProcessIncomming(acbuf & inBuf, string & sErrorRet)
 			ldbg("Peer wants to close connection after request");
 			m_bReconnectASAP=true;
 		}
-		p=h.h[header::TRANSFER_ENCODING];
-		if (p && 0==strcasecmp(p, "chunked"))
+		if(m_pStorage->m_bHeadOnly)
+			m_DlState=STATE_FINISHJOB;
+		else if ( NULL != (p=h.h[header::TRANSFER_ENCODING])
+				&& 0==strcasecmp(p, "chunked"))
 			m_DlState=STATE_GETCHUNKHEAD;
 		else
 		{
