@@ -8,11 +8,15 @@
 #include "acfg.h"
 #include "acbuf.h"
 #include "fileio.h"
+#include "cleaner.h"
 
 #include <errno.h>
+#include <algorithm>
 
 using namespace MYSTD;
 
+#define MAXTEMPDELAY 27
+
 header const & fileitem::GetHeaderUnlocked()
 {
 	return m_head;
@@ -26,18 +30,22 @@ string fileitem::GetHttpMsg()
 	return m_head.frontLine;
 }
 
+time_t fileitem::m_nEarliestExpiration(0);
+
 fileitem::fileitem(MYSTD::string sPath) :
 	condition(),
-	m_bCheckFreshness(true),
 	m_nIncommingCount(0),
 	m_nSizeSeen(0),
 	m_sPath(acfg::cachedir+SZPATHSEP+sPath),
 	m_sKey(sPath),
+	m_bCheckFreshness(true),
+	m_bHeadOnly(false),
+	m_bAllowStoreData(true),
 	m_nSizeChecked(0),
 	m_filefd(-1),
 	m_nDlRefsCount(0),
 	status(FIST_FRESH),
-	m_bAllowStoreData(true)
+	m_nTimeExpireAt(0)
 {
 
 	// not using hexmap, just converting only few allowed chars
@@ -131,29 +139,38 @@ int fileitem::GetFileFd() {
 	return fd;
 }
 
+off_t GetFileSize(cmstring & path, off_t defret)
+{
+	struct stat stbuf;
+	return (0==::stat(path.c_str(), &stbuf)) ? stbuf.st_size : defret;
+}
 
-FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
-	
+
+void fileitem::ResetCacheState()
+{
 	setLockGuard;
+	m_nSizeSeen = 0;
+	m_nSizeChecked = 0;
+	status = FIST_FRESH;
+	m_bAllowStoreData = true;
+	m_head.clear();
+}
+
+FiStatus fileitem::Setup(bool bCheckFreshness)
+{
+	setLockGuard;
+
 	if(status>FIST_FRESH)
 		return status;
-	
 	status=FIST_INITED;
-	struct stat stbuf;
-	
 	m_bCheckFreshness = bCheckFreshness;
 	
-	if(bIgnoreCached)
-		return status;
-	
 	if(m_head.LoadFromFile(m_sPath+".head") >0 && m_head.type==header::ANSWER )
 	{
 		if(200 != m_head.getStatus())
 			goto error_clean;
 		
-		if (0==::stat(m_sPath.c_str(), &stbuf))
-			m_nSizeSeen=stbuf.st_size;
-		
+		m_nSizeSeen=GetFileSize(m_sPath, 0);
 
 		// some plausibility checks
 		if(m_bCheckFreshness)
@@ -174,11 +191,12 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 				if(nContLen < m_nSizeSeen)
 					goto error_clean;
 				
-				// accept length for non-volatile files...
-				m_nSizeChecked=m_nSizeSeen;
-				// ... are they even complete? a 0 value also looks weird, try to continue
+				// is it complete? and 0 value also looks weird, try to verify
 				if(m_nSizeSeen == nContLen && nContLen>0)
-					status=FIST_COMPLETE;	
+				{
+					status=FIST_COMPLETE;
+					m_nSizeChecked=m_nSizeSeen;
+				}
 			}
 			else
 			{
@@ -189,13 +207,10 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 	}
 	else // -> no .head file
 	{
-		if(bCheckFreshness)
-		{
-			// maybe there is some left-over without head file? Don't thrust volatile data, but otherwise try to reuse?
-			// No, evil, deletes too fast and there is no way back if redownload fails unlink(m_sPath.c_str());
-		}
-		else if (0==::stat(m_sPath.c_str(), &stbuf))
-			m_nSizeSeen=stbuf.st_size;
+		// maybe there is some left-over without head file?
+		// Don't thrust volatile data, but otherwise try to reuse?
+		if(!bCheckFreshness)
+			m_nSizeSeen=GetFileSize(m_sPath, 0);
 	}
 	return status;
 
@@ -227,6 +242,9 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	if(status<FIST_DLGOTHEAD)
 		status=FIST_DLGOTHEAD; // assume that for now, may become error state...
 	
+	if(m_bCheckFreshness)
+		m_nTimeExpireAt = time(0) + MAXTEMPDELAY;
+
 	m_nIncommingCount+=h.m_nEstimLength;
 
 	// optional optimization: hints for the filesystem resp. kernel
@@ -235,24 +253,26 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
 
-#define SETERROR(x) { m_bAllowStoreData=false; m_head.frontLine=(tSS() << "HTTP/1.1 " << (x)); status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
+#define SETERROR(x) { m_bAllowStoreData=false; m_head.frontLine="HTTP/1.1 "; \
+	m_head.frontLine+=x; status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
 #define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
 #define BOUNCE(x) { SETERROR(x); return false; }
 
 	m_head.set(header::XORIG, h.h[header::XORIG]);
-	
-	int code=h.getStatus();
-	ldbg("http code: "<< code);
+
 	string sHeadPath=m_sPath+".head";
 	
-	if(200 == code)
+	switch(h.getStatus())
+	{
+	case 200:
 	{
 		m_nSizeChecked=0;
 		m_head=h;
 		const char *p=h.h[header::CONTENT_LENGTH];
 		hint_length=p ? atoofft(p) : 0;
+		break;
 	}
-	else if(206 == code)
+	case 206:
 	{
 		if(m_nSizeSeen<=0)
 		{
@@ -290,8 +310,8 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		m_head.del(header::CONTENT_RANGE);
 		m_head.set(header::CONTENT_LENGTH, mylen);
 
-		// special optimization... if "-1 trick" was used, don't reopen that file for writing later
-		if(m_bCheckFreshness && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
+		// special optimization; if "-1 trick" was used then maybe don't reopen that file for writing later
+		if(m_bCheckFreshness && pNextData && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
 		{
 			int fd=open(m_sPath.c_str(), O_RDONLY);
 			if(fd>=0)
@@ -299,7 +319,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 				if(m_nSizeChecked==lseek(fd, m_nSizeChecked, SEEK_SET))
 				{
 					char c;
-					if(1 == read(fd, &c, 1) && pNextData && c == *pNextData)
+					if(1 == read(fd, &c, 1) && c == *pNextData)
 					{
 						aclog::err("known data hit, don't write to...");
 						aclog::err(m_sPath);
@@ -310,22 +330,32 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 				forceclose(fd);
 			}
 		}
+		break;
 	}
-	else if(416 == code)
-	{
+	case 416:
 		// that's always bad; it cannot have been complete before (the -1 trick)
 		// -> kill cached file ASAP
 		m_bAllowStoreData=false;
 		SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
-	}
-	else
-	{
+		break;
+// use the Location header and be sure the server does not mess around with Location and other codes
+	case 301:
+	case 302:
+	case 303:
+	case 307:
+		m_head.set(header::LOCATION, h.h[header::LOCATION]);
+		// fall-through
+	default:
 		m_bAllowStoreData=false;
 		// have a clean header with just the error message
 		m_head.frontLine=h.frontLine;
 		m_head.type=header::ANSWER;
-		if(h.h[header::LOCATION] && (301==code||302==code||303==code||307==code))
-			m_head.set(header::LOCATION, h.h[header::LOCATION]);
+		m_head.set(header::CONTENT_LENGTH, "0");
+		if(status>FIST_DLGOTHEAD)
+		{
+			// oh shit. Client may have already started sending it. Prevent such trouble in future.
+			unlink(sHeadPath.c_str());
+		}
 	}
 
 	if(acfg::debug>1)
@@ -365,7 +395,8 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 					ldbg("file opened after copying around: ");
 				}
 				else
-					BOUNCE("503 Cannot store or remove files");
+					BOUNCE((tSS()<<"503 Cannot store or remove files in "
+							<< GetDirPart(m_sPath)).c_str());
 			}
 			else
 			{
@@ -376,8 +407,8 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		}
 		
 		if (m_filefd<0)
-			BOUNCE(errno==ENOSPC ? "503 OUT OF DISK SPACE" : 
-			"503 Cache storage error while opening data file");
+			BOUNCE( (errno==ENOSPC ? "503 OUT OF DISK SPACE" :
+			"503 Cache storage error while opening data file"));
 		
 		if(0!=fstat(m_filefd, &stbuf) || !S_ISREG(stbuf.st_mode))
 			SETERRORKILLFILE("503 Not a regular file");
@@ -405,7 +436,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		}
 
 		if(count<0)
-			SETERRORKILLFILE( (-count!=ENOSPC) ? "503 Cache storage error" : "503 OUT OF DISK SPACE");
+			SETERRORKILLFILE( (-count!=ENOSPC ? "503 Cache storage error" : "503 OUT OF DISK SPACE"));
 			
 		// double-check the sane state
 		if(0!=fstat(m_filefd, &stbuf) || stbuf.st_size!=m_nSizeChecked)
@@ -502,7 +533,8 @@ struct tFileRefEntry
 {
 	int nRefCount;
 	tFileItemPtr ref;
-	tFileRefEntry() : nRefCount(0) { };
+	bool m_bNeedsDelayedExpiration;
+	tFileRefEntry() : nRefCount(0), m_bNeedsDelayedExpiration(false) { };
 };
 static map<string, tFileRefEntry> mapItems;
 lockable mapLck;
@@ -563,13 +595,77 @@ tFileItemPtr fileitem::GetFileItem(MYSTD::string sPathKey)
 		}
 		return entry.ref;
 	}
-	MYCATCH(std::bad_alloc)
+	MYCATCH(MYSTD::bad_alloc&)
 	{
 		if(p)
 			p.reset();
 		return p;
 	}
+}
 
+
+// Opposite of GetFileItem, unregisters existing file item now or as soon as possible.
+void fileitem::UnregOneASAP(tFileItemPtr pItem)
+{
+	if (!pItem)
+		return;
+	// download never started or no delayed expiration needed -> release now. Or expired long ago.
+	if (!pItem->m_nTimeExpireAt || time(0) >= pItem->m_nTimeExpireAt)
+	{
+		pItem->Unreg();
+		return;
+	}
+	lockguard lockGlobalMap(mapLck);
+	std::map<string, tFileRefEntry>::iterator entry = mapItems.find(pItem->m_sKey);
+	// not existing? -> not for us. Still enough references? -> let this one go
+	if (mapItems.end() == entry || entry->second.nRefCount>1)
+	{
+		lockGlobalMap.unLock();
+		pItem->Unreg();
+		return;
+	}
+	// now the item's refCount is by one higher then it should be, set the flag instead of --
+	entry->second.m_bNeedsDelayedExpiration=true;
+
+// this will work as long as the delay is equal for all
+	m_nEarliestExpiration=pItem->m_nTimeExpireAt;
+
+}
+
+// this method is supposed to be awaken periodically and detect items with ref count manipulated by
+// the request storm prevention mechanism. Items shall be be dropped after some time if no other
+// thread but us is using them.
+time_t fileitem::DoDelayedUnregAndCheck()
+{
+	lockguard lockGlobalMap(mapLck);
+	std::map<string, tFileRefEntry>::iterator it, here;
+
+	time_t now=time(0);
+
+	if(now<m_nEarliestExpiration) // unreachable anyhow
+		return m_nEarliestExpiration;
+
+	m_nEarliestExpiration = cleaner::never; // the correct time will be found soon
+
+	for(it=mapItems.begin(); it!=mapItems.end();)
+	{
+		here=it++;
+		if(!here->second.m_bNeedsDelayedExpiration)
+			continue;
+		// still active or young item? skip, but find the earliest time to expire
+		if(now < here->second.ref->m_nTimeExpireAt || here->second.nRefCount>1)
+		{
+			m_nEarliestExpiration=min(here->second.ref->m_nTimeExpireAt, m_nEarliestExpiration);
+			continue;
+		}
+		// ok, unused and delay is over. Destroy with the same sequence as Unreg... does.
+		lockguard g(here->second.ref.get());
+		here->second.ref->status=FIST_ERRNOUSER;
+		here->second.ref->notifyAll();
+		g.unLock();
+		mapItems.erase(here);
+	}
+	return m_nEarliestExpiration;
 }
 
 #ifdef DEBUG
