@@ -10,7 +10,7 @@
 
 #include "md5.h"
 #include "sha1.h"
-
+#include "csmapping.h"
 
 #define FLAG_PLAIN 1
 #define FLAG_GZ 2
@@ -30,7 +30,9 @@ using namespace MYSTD;
 filereader::filereader() 
 :
 	flags(0),
-	m_szFileBuf((char*)MAP_FAILED), 
+	m_szFileBuf((char*)MAP_FAILED),
+	m_nBufSize(0),
+	m_nBufPos(0),
 	m_nCurLine(0),
 	m_fd(-1)
 {
@@ -122,11 +124,18 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 		goto ofail; 
     }
 	
-	m_nBufSize = statbuf.st_size;
-	m_szFileBuf = (char*) mmap(0, m_nBufSize, PROT_READ, MAP_SHARED, m_fd, 0);
-	
-	if(m_szFileBuf==MAP_FAILED)
-		goto ofail;
+	if(statbuf.st_size>0)
+	{
+		m_nBufSize = statbuf.st_size;
+		m_szFileBuf = (char*) mmap(0, m_nBufSize, PROT_READ, MAP_SHARED, m_fd, 0);
+		if(m_szFileBuf==MAP_FAILED)
+			goto ofail;
+	}
+	else
+	{
+		m_szFileBuf = NULL;
+		m_nBufSize = 0;
+	}
 	
 #ifdef HAVE_MADVISE
 	// if possible, prepare to read that
@@ -334,6 +343,7 @@ inline void filereader::_UncompressMoreData() {
 		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
 	}
 #endif
+#ifdef HAVE_LZMA
 	if(flags&FLAG_LZMA)
 	{
 		m_xzStream.next_in=(const uint8_t*)m_szFileBuf+m_nBufPos;
@@ -356,8 +366,6 @@ inline void filereader::_UncompressMoreData() {
 		// or corrupted data?
 		else flags|=(FLAG_COMPEOF|FLAG_ERROR);
 	}
-#ifdef HAVE_LZMA
-
 #endif
 }
 
@@ -375,8 +383,9 @@ bool filereader::GetMd5String(const MYSTD::string & sFileName, char out[])
 	return true;
 }
 */
-
-bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize)
+/*
+bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack,
+		off_t &scannedSize)
 {
 	md5_state_s ctx;
 	md5_init(&ctx);
@@ -411,30 +420,60 @@ bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool
 }
 
 bool filereader::GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], 
-		bool bTryUnpack, off_t &scannedSize, const char *szUnpStoreFile)
+		bool bTryUnpack, off_t &scannedSize, FILE *fDump)
 {
 	filereader f;
 	return (f.OpenFile(sFileName, !bTryUnpack)
-			&& f.GetSha1Sum(out, scannedSize, szUnpStoreFile));
+			&& f.GetSha1Sum(out, scannedSize, fDump));
+}
+*/
+
+
+class csumSHA1 : public csumBase, public SHA_INFO
+{
+public:
+	csumSHA1() { sha_init(this); }
+	void add(const char *data, size_t size) { sha_update(this, (SHA_BYTE*) data, size); }
+	void finish(uint8_t* ret) { sha_final(ret, this); }
+};
+class csumMD5 : public csumBase, public md5_state_s
+{
+public:
+	csumMD5() { md5_init(this); }
+	void add(const char *data, size_t size) { md5_append(this, (md5_byte_t*) data, size); }
+	void finish(uint8_t* ret) { md5_finish(this, ret); }
+};
+
+auto_ptr<csumBase> csumBase::GetChecker(CSTYPES type)
+{
+	switch(type)
+	{
+	case CSTYPE_MD5:
+		return auto_ptr<csumBase>(new csumMD5);
+	case CSTYPE_SHA1:
+	default: // for now
+		return auto_ptr<csumBase>(new csumSHA1);
+	}
 }
 
-
-bool filereader::GetSha1Sum(uint8_t out[], off_t &scannedSize, const char *szUnpStoreFile)
+bool filereader::GetChecksum(const mstring & sFileName, int csType, uint8_t out[],
+		bool bTryUnpack, off_t &scannedSize, FILE *fDump)
 {
-	SHA_INFO ctx;
-	sha_init(&ctx);
+	filereader f;
+	return (f.OpenFile(sFileName, !bTryUnpack)
+			&& f.GetChecksum(csType, out, scannedSize, fDump));
+}
+
+bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE *fDump)
+//bool filereader::GetSha1Sum(uint8_t out[], off_t &scannedSize, FILE *fDump)
+{
+	auto_ptr<csumBase> summer = csumBase::GetChecker(CSTYPES(csType));
 	scannedSize=0;
 	
-	FILE *fDump = NULL;
-	if(szUnpStoreFile)
-	{
-		fDump = fopen(szUnpStoreFile, "w");
-		if(!fDump)
-			return false;
-	}
 	if (flags&FLAG_PLAIN)
 	{
-		sha_update(&ctx, (SHA_BYTE*) m_szFileBuf, m_nBufSize);
+		summer->add(m_szFileBuf, m_nBufSize);
+		//sha_update(&ctx, (SHA_BYTE*) m_szFileBuf, m_nBufSize);
 		if(fDump)
 			fwrite(m_szFileBuf, sizeof(char), m_nBufSize, fDump);
 		scannedSize=m_nBufSize;
@@ -449,7 +488,8 @@ bool filereader::GetSha1Sum(uint8_t out[], off_t &scannedSize, const char *szUnp
 			UINT nRest=m_UncompBuf.size();
 			if (nRest==0)
 				break;
-			sha_update(&ctx, (md5_byte_t*) m_UncompBuf.rptr(), nRest);
+			summer->add(m_UncompBuf.rptr(), nRest);
+			//sha_update(&ctx, (md5_byte_t*) m_UncompBuf.rptr(), nRest);
 			if(fDump)
 				fwrite(m_UncompBuf.rptr(), sizeof(char), nRest, fDump);
 			scannedSize+=nRest;
@@ -457,61 +497,33 @@ bool filereader::GetSha1Sum(uint8_t out[], off_t &scannedSize, const char *szUnp
 		}
 
 	}
-	sha_final(out, &ctx);
-
-	if(fDump)
-	{
-		
-		// reliable closing
-		
-		int fd=fileno(fDump);
-		
-		for(int i=0; 0 != ::fflush(fDump) && i<10; i++) ;
-		
-		if(0!=::fclose(fDump))
-		{
-			if(errno == EBADF)
-				return false;
-			
-			while(0 != close(fd))
-			{
-				if(errno != EINTR)
-					return false;
-			}
-		}
-	}
+	//sha_final(out, &ctx);
+	summer->finish(out);
 	
 	return CheckGoodState(false);
 }
 
+// test checksum wrapper classes and their algorithms, also test conversion methods
 void check_algos()
 {
 	const char testvec[]="abc";
 	uint8_t out[20];
-	 
+	auto_ptr<csumBase> ap = csumBase::GetChecker(CSTYPE_SHA1);
+	ap->add(testvec, sizeof(testvec)-1);
+	ap->finish(out);
+	if(!CsEqual("a9993e364706816aba3e25717850c26c9cd0d89d", out, 20))
 	{
-		SHA_INFO ctx;
-		sha_init(&ctx);
-		sha_update(&ctx, (SHA_BYTE*) testvec, sizeof(testvec)-1);
-		sha_final(out, &ctx);
-		if (CsBinToString(out, 20)!="a9993e364706816aba3e25717850c26c9cd0d89d")
-		{
-			cerr
-					<< "Incorrect SHA1 implementation detected, check compilation settings!\n";
-			exit(EXIT_FAILURE);
-		}
+		cerr << "Incorrect SHA1 implementation detected, check compilation settings!\n";
+		exit(EXIT_FAILURE);
 	}
+
+	ap = csumBase::GetChecker(CSTYPE_MD5);
+	ap->add(testvec, sizeof(testvec) - 1);
+	ap->finish(out);
+	if (CsBinToString(out, 16) != "900150983cd24fb0d6963f7d28e17f72")
 	{
-		md5_state_s ctx;
-		md5_init(&ctx);
-		md5_append(&ctx, (md5_byte_t*) testvec, sizeof(testvec)-1);
-		md5_finish(&ctx, out);
-		if (CsBinToString(out, 16)!="900150983cd24fb0d6963f7d28e17f72")
-		{
-			cerr
-					<< "Incorrect MD5 implementation detected, check compilation settings!\n";
-			exit(EXIT_FAILURE);
-		}
+		cerr << "Incorrect MD5 implementation detected, check compilation settings!\n";
+		exit(EXIT_FAILURE);
 	}
 }
 
@@ -590,3 +602,39 @@ bool filereader::CheckMd5Sum(const string & sFileName, const string & sReference
 }
 
 */
+
+bool Bz2compressFile(const char *pathIn, const char*pathOut)
+{
+	bool bRet=false;
+	filereader reader;
+	FILE *f(NULL);
+	BZFILE *bzf(NULL);
+	int nError(0);
+
+	if(!reader.OpenFile(pathIn, true))
+		return false;
+
+	if(NULL !=(f = fopen(pathOut, "w")))
+	{
+		if(!ferror(f))
+		{
+			if(NULL != (bzf = BZ2_bzWriteOpen( &nError, f, 9, 0, 30)))
+			{
+				if(BZ_OK == nError)
+				{
+					BZ2_bzWrite(&nError, bzf, (void*) reader.GetBuffer(), reader.GetSize());
+					if(BZ_OK == nError)
+						bRet=true;
+				}
+				BZ2_bzWriteClose(&nError, bzf, 0, 0, 0);
+				if(BZ_OK != nError)
+					bRet=false;
+			}
+		}
+		if(ferror(f))
+			bRet=false;
+
+		checkForceFclose(f);
+	}
+	return bRet;
+}
