@@ -30,6 +30,7 @@ using namespace MYSTD;
 #define SPECIAL_FD -42
 inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
 
+
 /*
  * Unlike the regular store-and-forward file item handler, this ones does not store anything to
  * harddisk. Instead, it uses the download buffer and lets job object send the data straight from
@@ -47,13 +48,32 @@ public:
 	m_pData(NULL), m_nConsumable(0), m_nConsumed(0)
 	{
 		m_bAllowStoreData=false;
+		m_nSizeChecked = m_nSizeSeen = 0;
 	};
-	virtual FiStatus Setup(bool, bool bIgnoreCached=false)
+	virtual FiStatus Setup(bool)
 	{
 		return status = FIST_INITED;
+		m_nSizeChecked = m_nSizeSeen = 0;
 	}
-	virtual void Unreg() {}; // NOOP, since this item is not shared
+	virtual void Unreg() {
+		setLockGuard;
+		if(FIST_DLRECEIVING == status)
+		{
+			// no shit, there is some input waiting? bring the sender down ASAP
+			// this is not nice to the sender because it triggers a reconnection, but
+			// it's a local source so it shouldn't hurt much
+			status = FIST_ERRNOUSER;
+			notifyAll();
+		}
+	};
 	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
+	virtual bool DownloadStartedStoreHeader(const header & h, const char *)
+	{
+		setLockGuard;
+		m_head=h;
+		status=FIST_DLGOTHEAD;
+		return true;
+	}
 	virtual bool StoreFileData(const char *data, unsigned int size)
 	{
 		setLockGuard;
@@ -222,6 +242,7 @@ job::job(header *h, con *pParent) :
 	//m_bCloseCon(false),
 	m_bIsHttp11(false),
 	m_state(STATE_SEND_MAIN_HEAD),
+	m_backstate(STATE_TODISCON),
 	m_pReqHead(h),
 	m_nSendPos(0),
 	m_nRangeLast(OFFT_MAX-1), // range size should never overflow
@@ -247,8 +268,7 @@ job::~job()
 	
 	checkforceclose(m_filefd);
 
-	if (m_pItem)
-		m_pItem->Unreg();
+	fileitem::UnregOneASAP(m_pItem);
 
 	delete m_pReqHead;
 }
@@ -540,7 +560,7 @@ void job::PrepareDownload() {
 			}
 		}
 
-		for(tStringMap::const_iterator it=acfg::localdirs.find(tUrl.sHost);
+		for(tStrMap::const_iterator it=acfg::localdirs.find(tUrl.sHost);
 				it != acfg::localdirs.end() ; )
 		{
 			string realSubPath;
@@ -584,7 +604,7 @@ void job::PrepareDownload() {
 #endif
 		}
 	}
-	MYCATCH(out_of_range) // better safe...
+	MYCATCH(MYSTD::out_of_range&) // better safe...
 	{
     	goto report_invpath;
     }
@@ -616,10 +636,11 @@ void job::PrepareDownload() {
 	if (bPtMode && fistate != FIST_COMPLETE)
 		fistate = _SwitchToPtItem(m_sFileLoc);
 
+	if(fistate==FIST_COMPLETE)
+		return; // perfect, done here
+
     if(acfg::offlinemode) { // make sure there will be no problems later in SendData or prepare a user message
-    	if(fistate==FIST_COMPLETE)
-    		return; // perfect, done here
-    	// error and/or needs download, but freshness check was disabled, so it's really not complete.
+    	// error or needs download but freshness check was disabled, so it's really not complete.
     	goto report_offlineconf;
     }
     dbgline;
@@ -659,7 +680,7 @@ void job::PrepareDownload() {
 			}
 			ldbg("Download job enqueued for " << m_sFileLoc);
 		}
-		MYCATCH(std::bad_alloc) // OOM, may this ever happen here?
+		MYCATCH(MYSTD::bad_alloc&) // OOM, may this ever happen here?
 		{
 			if(acfg::debug)
 				aclog::err("Out of memory");			    		
@@ -913,7 +934,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					return R_DISCON;
 			}
 		}
-		MYCATCH(bad_alloc) {
+		MYCATCH(bad_alloc&) {
 			// TODO: report memory failure?
 			return R_DISCON;
 		}
@@ -939,7 +960,7 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 	bool bHasSendableData = false;
 	int httpstatus = respHead.getStatus();
 	LOG("State: " << httpstatus);
-	if (304 != httpstatus) // never for not-unmodified
+	if (!BODYFREECODE(httpstatus)) // never for not-unmodified&co.
 	{
 		bool bChunked = respHead.h[header::TRANSFER_ENCODING] && 0 == strncasecmp( respHead.h[header::TRANSFER_ENCODING], "chunk", 5);
 		if (httpstatus == 200)
@@ -1122,6 +1143,7 @@ FiStatus job::_SwitchToPtItem(const MYSTD::string &fileLoc)
 	return m_pItem->Setup(true);
 }
 
+
 void job::SetErrorResponse(const char * errorLine, int nMinUserLogLevel)
 {
 	LOGSTART2("job::SetErrorResponse", errorLine);
@@ -1132,8 +1154,7 @@ void job::SetErrorResponse(const char * errorLine, int nMinUserLogLevel)
 	public:
 		erroritem(const string &sId, const char *szError) : tGeneratedFitemBase(sId, szError)
 		{
-			// not-modified -> no response
-			if(304 == m_head.getStatus())
+			if(BODYFREECODE(m_head.getStatus()))
 				return;
 			// otherwise do something meaningful
 			m_data << "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n"
@@ -1146,8 +1167,7 @@ void job::SetErrorResponse(const char * errorLine, int nMinUserLogLevel)
 
 	tFileItemPtr pNew(static_cast<fileitem*>(new erroritem("noid", errorLine)));
 
-	if(m_pItem)
-		m_pItem->Unreg();
+	fileitem::UnregOneASAP(m_pItem);
 
 	m_pItem=pNew;
 
