@@ -12,15 +12,16 @@
 #include "csmapping.h"
 
 #include <string>
-//#include <iostream>
+#include <iostream>
+#include <fstream>
+#include <meta.h>
 #include <cstdio>
 #include <errno.h>
 
 using namespace MYSTD;
 
-#define SCACHEFILE (acfg::cachedir+SZPATHSEP"_impkeycache")
-extern bool bSigTaskAbort;
-extern pthread_mutex_t abortMx;
+#define SCACHEFILE (CACHE_BASE+"_impkeycache")
+#define FMTSIG "FMT4"
 
 /*
  * Algorithm:
@@ -33,38 +34,22 @@ extern pthread_mutex_t abortMx;
  *
 */
 
-pkgimport::pkgimport(int fd) : expiration(fd), m_bLookForIFiles(false)
+pkgimport::pkgimport(int fd) : tCacheMan(fd), m_bLookForIFiles(false)
 {
+	m_sTypeName="File Import";
 }
 
-bool pkgimport::ProcessOther(const string &sPath, const struct stat &stinfo) 
-{
-	// NOOP;
-	return true;
-}
-
-// Evil stuff. Tries to guess the appropriate checksum type 
-// and returns a pointer to static parametrized fingerprint object
-inline tFingerprint *CsTypeGuessing(const string & sPath)
+/*
+inline bool IsIndexDiff(const string & sPath)
 {
 	static tFingerprint fprBuf;
-
 	static string sDiffHint(".diff" SZPATHSEP);
 	tStrPos pos;
-	if (endsWithSzAr(sPath,".gz") && ( sPath.find(sDiffHint)!=stmiss ||
-			( stmiss!=(pos=sPath.rfind(SZPATHSEP "20")) && sPath.find_first_not_of("0123456789.-", pos+3)==sPath.size()-2)))
-	{
-		fprBuf.csType=CSTYPE_SHA1;
-		fprBuf.bUnpack=true;
-	}
-	else
-	{
-		fprBuf.csType=CSTYPE_MD5;
-		fprBuf.bUnpack=false;
-	}
-	return &fprBuf;
+	return (endsWithSzAr(sPath,".gz") && (sPath.find(sDiffHint) != stmiss || (stmiss != (pos
+			= sPath.rfind(SZPATHSEP "20")) && sPath.find_first_not_of("0123456789.-", pos + 3)
+			== sPath.size() - 2)));
 }
-
+*/
 
 bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 {
@@ -76,36 +61,39 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 	}
 	if(endsWithSzAr(sPath, ".head"))
 		return true;
+
+	if(sPath.size()<=CACHE_BASE_LEN) // heh?
+		return false;
 	
 	if(m_bLookForIFiles)
 	{
-		if(0==sPath.compare(m_nDropPrefLen, 1, "_"))
+		if(0==sPath.compare(CACHE_BASE_LEN, 1, "_"))
 			return true; // not for us, also includes _import
-		
-		AddIFileCandidate(sPath);
+
+		AddIFileCandidate(sPath.substr(CACHE_BASE_LEN));
 	}
-	else
+	else if(rechecks::FILE_INVALID != rechecks::GetFiletype(sPath))
 	{
 		// get a fingerprint by checksumming if not already there from the fpr cache
 
 		if(m_precachedList.find(sPath)!=m_precachedList.end())
 			return true; // have that already, somewhere...
 		
-		tFingerprint *fpr=CsTypeGuessing(sPath);
-		if (fpr->ReadCsFromFile(sPath))
-		{
-			SendChunk(string("<font color=blue>Checked ")+sPath
-					+" (fingerprint created)</font><br>\n");
-		}
+		// get the most likely requested contents id
+		tFingerprint fpr;
+	/*	if ( (IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_SHA1, true))
+				|| (!IsIndexDiff(sPath) && fpr.ScanFile(sPath, CSTYPE_MD5, false)))
+		*/
+		if( fpr.ScanFile(sPath, CSTYPE_MD5, false))
+			SendFmt()<<"<font color=blue>Checked " << sPath<<" (fingerprint created)</font><br>\n";
 		else
 		{
-			SendChunk(string("<font color=red>Error checking ")+sPath
-					+"</font><br>\n");
+			SendFmt()<< "<font color=red>Error checking " << sPath << "</font><br>\n";
 			return true;
 		}
 		
 		// add this entry immediately if needed and get its reference
-		tImpFileInfo & node = m_importMap[*fpr];
+		tImpFileInfo & node = m_importMap[fpr];
 		if (node.sPath.empty())
 		{ // fresh, just added to the map
 			node.sPath=sPath;
@@ -113,10 +101,16 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 		}
 		else if(node.sPath!=sPath)
 		{
-			SendChunk(string("<font color=orange>Duplicate found, ")+sPath
-								+" vs. "+node.sPath+", ignoring new entry.</font><br>\n");
+			SendFmt() << "<font color=orange>Duplicate found, " << sPath
+					<< " vs. " << node.sPath << ", ignoring new entry.</font><br>\n";
+			m_importRest.push_back(make_pair(fpr, tImpFileInfo(sPath, stinfo.st_mtime)));
 		}
 	}
+	else
+	{
+		if(m_bVerbose)
+			SendFmt() << "<font color=orange>File type unknown, skipping "+sPath << "</font><br>\n";
+	}
 	return true;
 }
 
@@ -172,6 +166,7 @@ bool FileCopy(const string &from, const string to)
 	checkforceclose(out);
 	return false;
 }
+
 bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 {
 	mkbasedir(to);
@@ -208,7 +203,7 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 		if (0==link(from.c_str(), to.c_str()))
 			return true;
 
-		SendChunk("(couldn't link, copying contents)");
+		SendChunk("(couldn't link, copying contents)<br>\n");
 		if(!FileCopy(from,to))
 			return false;
 		
@@ -226,10 +221,14 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 void pkgimport::Action(const string &cmd) 
 {
 // TODO: import path from cmd?
-	m_sSrcPath=acfg::cachedir+SZPATHSEP"_import";
+	m_sSrcPath=CACHE_BASE+"_import";
 	
 	SendChunk(string("Importing from ")+m_sSrcPath+" directory, scanning...<br>\n");
 	
+	SetCommonUserFlags(cmd);
+	m_bErrAbort=false; // does not f...ing matter, do what we can
+	m_bByPath=true; // should act on all locations
+
 	_LoadKeyCache(SCACHEFILE);
 	if(!m_precachedList.empty())
 		SendChunk( tSS(100)<<"Loaded "<<m_importMap.size()<<
@@ -245,12 +244,13 @@ void pkgimport::Action(const string &cmd)
 			return;
 	}
 	
-	if(m_indexFiles.empty())
+	if(m_indexFilesRel.empty())
 	{
 		SendChunk("<font size=0 color=red>No index files detected. Unable to continue, cannot map files to internal locations.</font>");
 		return;
 	}
 
+
 	UpdateIndexFiles();
 
 	{
@@ -287,15 +287,19 @@ void pkgimport::Action(const string &cmd)
 		if(bSigTaskAbort)
 			return;
 	}
-	
-	FILE *cfh = fopen(SCACHEFILE.c_str(), "wb");
-	if (!cfh)
+
+	ofstream fList;
+	fList.open(SCACHEFILE.c_str(), ios::out | ios::trunc);
+
+	if(!fList.is_open())
 	{
 		unlink(SCACHEFILE.c_str());
-		cfh=fopen(SCACHEFILE.c_str(), "wb");
+		fList.open(SCACHEFILE.c_str(), ios::out| ios::trunc);
 	}
-	if(cfh)
-		fprintf(cfh, "FMT3\n");
+	if(!fList.is_open())
+		SendChunk("Cannot save fingerprint cache, ignored");
+	else
+		fList << FMTSIG"\n";
 	
 	for(tImportMap::const_iterator it=m_importMap.begin();
 	it!=m_importMap.end();it++)
@@ -303,52 +307,61 @@ void pkgimport::Action(const string &cmd)
 		// delete imported stuff, and cache the fingerprint only when file was deleted
 		
 		if(it->second.bFileUsed && 0==unlink(it->second.sPath.c_str()))
-				continue;
-		
-		// deal with unpacked checksums from pdiffs? don't care for now
-		if(cfh)
-		{
-			fprintf(cfh, 
-					"%"PRIu64"\n%d\n%d\n%s\n"
-					"%s\n%"PRIu64"\n\n", 
-					(uint64_t) it->first.size,
-					(int) it->first.csType,
-					(int) it->first.bUnpack,
-					it->first.GetCsAsString().c_str(),
-					it->second.sPath.c_str(),
-					(uint64_t) it->second.mtime	);
-		}
+			continue;
+		if(!fList.is_open())
+			continue;
+
+		// not caching sha1-checksums of gzipped index files, less cumbersome and hardly required
+		if(!it->first.csType == CSTYPE_MD5)
+			continue;
+#define ENDL "\n" // don't flush all the time
+		fList << it->first.size << ENDL
+				<< it->first.GetCsAsString() << ENDL
+				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
+				<< it->second.mtime	<<ENDL;
 	}
-	checkForceFclose(cfh);
+
+	// deal with the rest, never double-examine that stuff
+	// copy&paste FTW
+	for(tImportList::const_iterator it=m_importRest.begin();
+	it!=m_importRest.end();it++)
+	{
+		// not caching sha1-checksums of gzipped index files, less cumbersome and hardly required
+		if(!it->first.csType == CSTYPE_MD5)
+			continue;
+		fList << it->first.size << ENDL
+				<< it->first.GetCsAsString() << ENDL
+				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
+				<< it->second.mtime	<<ENDL;
+	}
+	fList.flush();
+	fList.close();
 }
 
-void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
+void pkgimport::HandlePkgEntry(const tRemoteFileInfo &entry, bool)
 {
-	
-	//if(entry.sFileName.find("unp_1.0.15.tar.gz")!=stmiss)
-	//	int wtf=1;
 	//typedef MYSTD::map<tFingerprint, tImpFileInfo, ltfingerprint> tImportMap;
 	tImportMap::iterator hit = m_importMap.find(entry.fpr);
 	if (hit==m_importMap.end())
 		return;
 	
-	string sDest=entry.sDirectory+entry.sFileName;
-	string sDestHead=sDest+".head";
-	const string &sFrom=hit->second.sPath;
+	string sDestAbs=CACHE_BASE+entry.sDirectory+entry.sFileName;
+	string sDestHeadAbs=sDestAbs+".head";
+	const string &sFromAbs=hit->second.sPath;
 
-	SendChunk(string("<font color=green>HIT: ")+sFrom
-			+ "<br>\nDESTINATION: "+sDest+"</font><br>\n");
+	SendChunk(string("<font color=green>HIT: ")+sFromAbs
+			+ "<br>\nDESTINATION: "+sDestAbs+"</font><br>\n");
 
 	// linking and moving would shred them when the link leads to the same target
 	struct stat tmp1, tmp2;
 
-	if (0==stat(sDest.c_str(), &tmp1) && 0==stat(sFrom.c_str(), &tmp2)
+	if (0==stat(sDestAbs.c_str(), &tmp1) && 0==stat(sFromAbs.c_str(), &tmp2)
 			&& tmp1.st_ino==tmp2.st_ino&& tmp1.st_dev==tmp2.st_dev)
 	{
 		//cerr << "Same target file, ignoring."<<endl;
 		hit->second.bFileUsed=true;
 		SendChunk("<font color=orange>Same file exists</font><br>\n");
-		if(0!=access(sDestHead.c_str(), F_OK))
+		if(0!=access(sDestHeadAbs.c_str(), F_OK))
 		{
 			SendChunk("<font color=orange>Header is missing, will recreate...</font><br>\n");
 			goto gen_header;
@@ -356,19 +369,20 @@ void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
 		return;
 	}
 
-	unlink(sDest.c_str());
+	unlink(sDestAbs.c_str());
 	
-	if (!LinkOrCopy(sFrom, sDest))
+	if (!LinkOrCopy(sFromAbs, sDestAbs))
 	{
 		SendChunk("<font color=red>ERROR: couldn't link or copy file.</font><br>\n");
 		return;
 	}
 
 	gen_header:
-	unlink(sDestHead.c_str());
+	unlink(sDestHeadAbs.c_str());
 
 	header h;
 	h.frontLine="HTTP/1.1 200 OK";
+	/*
 	if(entry.fpr.bUnpack)
 	{
 		static struct stat stbuf;
@@ -377,10 +391,11 @@ void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
 		h.set(header::CONTENT_LENGTH, stbuf.st_size);
 	}
 	else
+	*/
 		h.set(header::CONTENT_LENGTH, entry.fpr.size);
 	
 	h.type=header::ANSWER;
-	if (h.StoreToFile(sDest+".head")<=0)
+	if (h.StoreToFile(sDestAbs+".head")<=0)
 	{
 		aclog::err("Unable to store generated header");
 		return; // junk may remain but that's a job for cleanup later
@@ -391,58 +406,55 @@ void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
 
 void pkgimport::_LoadKeyCache(const string & sFileName)
 {
-	filereader reader;
-	string sLine;
-	if(!reader.OpenFile(sFileName)||!reader.GetOneLine(sLine)||sLine!="FMT3")
-		return;
-
-	/*typedef MYSTD::map<tFingerprint, tImpFileInfo, ltfingerprint> tImportMap;
-
-	fprintf(cfh, 
-			"%"PRIu64"\n%d\n%d\n%s\n"
-			"%s\n%"PRIu64"\n", 
-			(uint64_t) it->first.size,
-			(int) it->first.csType,
-			(int) it->first.bUnpack,
-			it->first.GetCsAsString().c_str(),
-			it->second.sPath.c_str(),
-			(uint64_t) it->second.mtime	);
-	*/
-
-#define _getline { if(!reader.GetOneLine(sLine)) break; }
+	std::ifstream in;
 
 	tImpFileInfo info;
 	tFingerprint fpr;
 	
-	while(true)
+	in.open(SCACHEFILE.c_str());
+	string cs;
+	if(!in.is_open())
+		return;
+
+	std::getline(in, cs);
+	if(cs!=FMTSIG)
+		return;
+
+/*	if(m_bVerbose)
+		SendChunk("Loading fingerprints from key cache... \n");
+		*/
+	for(;;)
 	{
 		info.bFileUsed=false;
-		_getline
-		fpr.size=atoofft(sLine.c_str());
-		_getline
-		fpr.csType = (CSTYPES) atoi(sLine.c_str());
-		_getline
-		fpr.bUnpack=(0!=atoi(sLine.c_str()));
-		_getline
-		fpr.ReadCsFromString(sLine);
-		_getline
-		info.sPath=sLine;
-		_getline
-		info.mtime=atol(sLine.c_str());
-		_getline
-		if(!sLine.empty())
-			break;
-			
+		in>>fpr.size;
+		std::getline(in, cs); // newline
+		std::getline(in, cs); // next line
+		fpr.Set(cs, CSTYPE_MD5, fpr.size);
+		std::getline(in, info.sPath);
+		info.sPath.insert(0, m_sSrcPath+SZPATHSEP);
+		in>>info.mtime;
+		std::getline(in, cs); // newline
+
+#ifdef DEBUG
+		bool bNix=(stmiss != info.sPath.find("cabextract"));
+#endif
+
 		struct stat stbuf;
 		if(0==stat(info.sPath.c_str(), &stbuf) 
-				&& info.mtime==stbuf.st_mtime
-				&& (fpr.size==stbuf.st_size || fpr.bUnpack))
+				&& info.mtime==stbuf.st_mtime)
 		{
 			m_importMap[fpr]=info;
 			m_precachedList.insert(info.sPath);
 		}
 		
+		if(in.eof() || in.fail())
+			break;
 	}
+	/*
+	 *
+	 if(m_bVerbose)
+		SendChunk(ltos(m_importMap.size())+" entries<br>\n");
+		*/
 }
 
 
